@@ -40,9 +40,9 @@ from .ui import emblemcellrenderer
 
 from collections import namedtuple
 from decimal import Decimal
-from gettext import gettext as _
 
-from meld.settings import meldsettings, settings
+from meld.conf import _
+from meld.settings import bind_settings, meldsettings, settings
 
 
 ################################################################################
@@ -248,6 +248,14 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
 
     __gtype_name__ = "DirDiff"
 
+    __gsettings_bindings__ = (
+        ('folder-ignore-symlinks', 'ignore-symlinks'),
+        ('folder-shallow-comparison', 'shallow-comparison'),
+        ('folder-time-resolution', 'time-resolution'),
+        ('folder-status-filters', 'status-filters'),
+        ('ignore-blank-lines', 'ignore-blank-lines'),
+    )
+
     ignore_blank_lines = GObject.property(
         type=bool,
         nick="Ignore blank lines",
@@ -302,11 +310,11 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         melddoc.MeldDoc.__init__(self)
         gnomeglade.Component.__init__(self, "dirdiff.ui", "dirdiff",
                                       ["DirdiffActions"])
+        bind_settings(self)
 
         self.ui_file = gnomeglade.ui_file("dirdiff-ui.xml")
         self.actiongroup = self.DirdiffActions
         self.actiongroup.set_translation_domain("meld")
-        self.main_actiongroup = None
 
         self.name_filters = []
         self.text_filters = []
@@ -408,17 +416,6 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         settings.connect('changed::folder-columns',
                          self.update_treeview_columns)
 
-        settings.bind('folder-ignore-symlinks', self, 'ignore-symlinks',
-                      Gio.SettingsBindFlags.DEFAULT)
-        settings.bind('folder-shallow-comparison', self, 'shallow-comparison',
-                      Gio.SettingsBindFlags.DEFAULT)
-        settings.bind('folder-time-resolution', self, 'time-resolution',
-                      Gio.SettingsBindFlags.DEFAULT)
-        settings.bind('folder-status-filters', self, 'status-filters',
-                      Gio.SettingsBindFlags.DEFAULT)
-        settings.bind('ignore-blank-lines', self, 'ignore-blank-lines',
-                      Gio.SettingsBindFlags.DEFAULT)
-
         self.update_comparator()
         self.connect("notify::shallow-comparison", self.update_comparator)
         self.connect("notify::time-resolution", self.update_comparator)
@@ -513,8 +510,6 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.filter_menu_button.set_label_widget(label)
 
     def on_container_switch_in_event(self, ui):
-        self.main_actiongroup = [a for a in ui.get_action_groups()
-                                 if a.get_name() == "MainActions"][0]
         melddoc.MeldDoc.on_container_switch_in_event(self, ui)
         self._create_filter_menu_button(ui)
         self.ui_manager = ui
@@ -1217,7 +1212,8 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             is_present = [ os.path.exists( f ) for f in curfiles ]
             all_present = 0 not in is_present
             if all_present:
-                if self.file_compare(curfiles, regexes) in (Same, SameFiltered):
+                if self.file_compare(curfiles, regexes) in (
+                        Same, SameFiltered, DodgySame):
                     state = tree.STATE_NORMAL
                 else:
                     state = tree.STATE_MODIFIED
