@@ -20,7 +20,6 @@ import copy
 import functools
 import io
 import os
-from gettext import gettext as _
 import time
 
 from multiprocessing import Pool
@@ -33,6 +32,7 @@ from gi.repository import Gio
 from gi.repository import Gdk
 from gi.repository import Gtk
 
+from meld.conf import _
 from . import diffutil
 from . import matchers
 from . import meldbuffer
@@ -45,7 +45,8 @@ from . import undo
 from .ui import findbar
 from .ui import gnomeglade
 
-from meld.settings import meldsettings, settings
+from meld.const import MODE_REPLACE, MODE_DELETE, MODE_INSERT
+from meld.settings import bind_settings, meldsettings, settings
 from .util.compat import text_type
 from meld.sourceview import LanguageManager
 
@@ -77,7 +78,10 @@ class CachedSequenceMatcher(object):
     def match(self, text1, textn, cb):
         try:
             self.cache[(text1, textn)][1] = time.time()
-            cb(self.cache[(text1, textn)][0])
+            # FIXME: This idle should be totally unnecessary, and yet nothing
+            # gets highlighted without it, even though everything in the
+            # callback appears to run identically.
+            GLib.idle_add(lambda: cb(self.cache[(text1, textn)][0]))
         except KeyError:
             def inline_cb(opcodes):
                 self.cache[(text1, textn)] = [opcodes, time.time()]
@@ -100,7 +104,6 @@ class CachedSequenceMatcher(object):
 
 
 MASK_SHIFT, MASK_CTRL = 1, 2
-MODE_REPLACE, MODE_DELETE, MODE_INSERT = 0, 1, 2
 
 
 class CursorDetails(object):
@@ -138,6 +141,11 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
 
     __gtype_name__ = "FileDiff"
 
+    __gsettings_bindings__ = (
+        ('highlight-current-line', 'highlight-current-line'),
+        ('ignore-blank-lines', 'ignore-blank-lines'),
+    )
+
     highlight_current_line = GObject.property(type=bool, default=False)
     ignore_blank_lines = GObject.property(
         type=bool,
@@ -168,17 +176,20 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         """
         melddoc.MeldDoc.__init__(self)
         gnomeglade.Component.__init__(self, "filediff.ui", "filediff")
+        bind_settings(self)
+
         widget_lists = [
             "diffmap", "file_save_button", "file_toolbar", "fileentry",
             "linkmap", "msgarea_mgr", "readonlytoggle",
             "scrolledwindow", "selector_hbox", "textview", "vbox",
-            "dummy_toolbar_linkmap"
+            "dummy_toolbar_linkmap", "filelabel_toolitem", "filelabel",
+            "fileentry_toolitem",
         ]
         self.map_widgets_into_lists(widget_lists)
 
         # This SizeGroup isn't actually necessary for FileDiff; it's for
         # handling non-homogenous selectors in FileComp. It's also fragile.
-        column_sizes = Gtk.SizeGroup(Gtk.SizeGroupMode.HORIZONTAL)
+        column_sizes = Gtk.SizeGroup(mode=Gtk.SizeGroupMode.HORIZONTAL)
         column_sizes.set_ignore_hidden(True)
         for widget in self.selector_hbox:
             column_sizes.add_widget(widget)
@@ -193,6 +204,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             buf.data.connect('file-changed', self.notify_file_changed)
         self._keymask = 0
         self.load_font()
+        self.meta = {}
         self.deleted_lines_pending = -1
         self.textview_overwrite = 0
         self.focus_pane = None
@@ -305,11 +317,10 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         )
 
         self.ui_file = gnomeglade.ui_file("filediff-ui.xml")
-        self.actiongroup = Gtk.ActionGroup('FilediffPopupActions')
+        self.actiongroup = Gtk.ActionGroup(name="FilediffPopupActions")
         self.actiongroup.set_translation_domain("meld")
         self.actiongroup.add_actions(actions)
         self.actiongroup.add_toggle_actions(toggle_actions)
-        self.main_actiongroup = None
 
         self.findbar = findbar.FindBar(self.grid)
         self.grid.attach(self.findbar.widget, 1, 2, 5, 1)
@@ -357,29 +368,6 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 gutter = t.get_gutter(window)
                 gutter.insert(renderer, 10)
 
-        # GSettings bindings
-        for view in self.textview:
-            settings.bind('indent-width', view, 'indent-width',
-                          Gio.SettingsBindFlags.DEFAULT)
-            settings.bind('insert-spaces-instead-of-tabs', view,
-                          'insert-spaces-instead-of-tabs',
-                          Gio.SettingsBindFlags.DEFAULT)
-            settings.bind('show-line-numbers', view, 'show-line-numbers',
-                          Gio.SettingsBindFlags.DEFAULT)
-            settings.bind('draw-spaces', view, 'draw-spaces',
-                          Gio.SettingsBindFlags.DEFAULT)
-            settings.bind('wrap-mode', view, 'wrap-mode',
-                          Gio.SettingsBindFlags.DEFAULT)
-
-        for buf in self.textbuffer:
-            settings.bind('highlight-syntax', buf, 'highlight-syntax',
-                          Gio.SettingsBindFlags.DEFAULT)
-
-        settings.bind('highlight-current-line', self, 'highlight-current-line',
-                      Gio.SettingsBindFlags.DEFAULT)
-        settings.bind('ignore-blank-lines', self, 'ignore-blank-lines',
-                      Gio.SettingsBindFlags.DEFAULT)
-
         self.connect("notify::ignore-blank-lines", self.refresh_comparison)
 
         meldsettings.connect('changed', self.on_setting_changed)
@@ -444,12 +432,6 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
     def on_focus_change(self):
         self.keymask = 0
 
-    def on_container_switch_in_event(self, ui):
-        self.main_actiongroup = [a for a in ui.get_action_groups()
-                                 if a.get_name() == "MainActions"][0]
-        melddoc.MeldDoc.on_container_switch_in_event(self, ui)
-        # FIXME: If no focussed textview, action sensitivity will be unset
-
     def on_text_filters_changed(self, app):
         relevant_change = self.create_text_filters()
         if relevant_change:
@@ -785,16 +767,26 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.textview[new_pane].scroll_to_mark(
             new_buf.get_insert(), 0.1, True, 0.5, 0.5)
 
+    def _set_external_action_sensitivity(self):
+        have_file = self.focus_pane is not None
+        try:
+            self.main_actiongroup.get_action("OpenExternal").set_sensitive(
+                have_file)
+        except AttributeError:
+            pass
+
     def on_textview_focus_in_event(self, view, event):
         self.focus_pane = view
         self.findbar.textview = view
         self.on_cursor_position_changed(view.get_buffer(), None, True)
         self._set_save_action_sensitivity()
         self._set_merge_action_sensitivity()
+        self._set_external_action_sensitivity()
         self.update_text_actions_sensitivity()
 
     def on_textview_focus_out_event(self, view, event):
         self._set_merge_action_sensitivity()
+        self._set_external_action_sensitivity()
 
     def _after_text_modified(self, buffer, startline, sizechange):
         if self.num_panes > 1:
@@ -889,6 +881,26 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                             return Gtk.ResponseType.CANCEL
             elif response == Gtk.ResponseType.DELETE_EVENT:
                 response = Gtk.ResponseType.CANCEL
+
+        if response == Gtk.ResponseType.OK and self.meta:
+            parent = self.meta.get('parent', None)
+            saved = self.meta.get('middle_saved', False)
+            prompt_resolve = self.meta.get('prompt_resolve', False)
+            if prompt_resolve and saved and parent.has_command('resolve'):
+                primary = _("Mark conflict as resolved?")
+                secondary = _(
+                    "If the conflict was resolved successfully, you may mark "
+                    "it as resolved now.")
+                buttons = ((_("Cancel"), Gtk.ResponseType.CANCEL),
+                           (_("Mark _Resolved"), Gtk.ResponseType.OK))
+                resolve_response = misc.modal_dialog(
+                    primary, secondary, buttons, parent=self.widget,
+                    messagetype=Gtk.MessageType.QUESTION)
+
+                if resolve_response == Gtk.ResponseType.OK:
+                    conflict_file = self.textbuffer[1].data.filename
+                    parent.command('resolve', [conflict_file])
+
         return response
 
     def on_delete_event(self, appquit=0):
@@ -1033,9 +1045,12 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             return
         buf = self.textbuffer[1]
         buf.data.savefile = os.path.abspath(filename)
-        buf.data.set_label(filename)
-        self.set_buffer_writable(buf, os.access(buf.data.savefile, os.W_OK))
-        self.fileentry[1].set_filename(os.path.abspath(filename))
+        buf.data.label = filename
+        writable = True
+        if os.path.exists(buf.data.savefile):
+            writable = os.access(buf.data.savefile, os.W_OK)
+        self.set_buffer_writable(buf, writable)
+        self.fileentry[1].set_filename(buf.data.savefile)
         self.recompute_label()
 
     def _set_save_action_sensitivity(self):
@@ -1058,7 +1073,11 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             self.file_save_button[i].props.stock_id = (
                 Gtk.STOCK_SAVE if buf.data.writable else Gtk.STOCK_SAVE_AS)
 
-        self.label_text = (" — ").decode('utf8').join(shortnames)
+        label = self.meta.get("tablabel", "")
+        if label:
+            self.label_text = label
+        else:
+            self.label_text = (" — ").decode('utf8').join(shortnames)
         self.tooltip_text = self.label_text
         self.label_changed()
 
@@ -1163,7 +1182,10 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                         t.buf.insert(t.buf.get_end_iter(), nextbit)
                     else:
                         if t.buf.data.savefile:
-                            writable = os.access(t.buf.data.savefile, os.W_OK)
+                            writable = True
+                            if os.path.exists(t.buf.data.savefile):
+                                writable = os.access(
+                                    t.buf.data.savefile, os.W_OK)
                         else:
                             writable = os.access(t.filename, os.W_OK)
                         self.set_buffer_writable(t.buf, writable)
@@ -1226,6 +1248,16 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         for i in self._diff_files():
             yield i
 
+    def set_meta(self, meta):
+        self.meta = meta
+        labels = meta.get('labels', ())
+        if labels:
+            for i, l in enumerate(labels):
+                if l:
+                    self.filelabel[i].set_text(l)
+                    self.filelabel_toolitem[i].set_visible(True)
+                    self.fileentry_toolitem[i].set_visible(False)
+
     def notify_file_changed(self, data):
         try:
             pane = [b.data for b in self.textbuffer].index(data)
@@ -1332,16 +1364,21 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                     self._prompt_long_highlighting()
                     continue
 
-                def apply_highlight(bufs, tags, starts, ends, texts, matches):
-                    starts = [bufs[0].get_iter_at_mark(starts[0]),
-                              bufs[1].get_iter_at_mark(starts[1])]
-                    ends = [bufs[0].get_iter_at_mark(ends[0]),
-                            bufs[1].get_iter_at_mark(ends[1])]
+                def apply_highlight(bufs, tags, start_marks, end_marks, texts, matches):
+                    starts = [bufs[0].get_iter_at_mark(start_marks[0]),
+                              bufs[1].get_iter_at_mark(start_marks[1])]
+                    ends = [bufs[0].get_iter_at_mark(end_marks[0]),
+                            bufs[1].get_iter_at_mark(end_marks[1])]
                     text1 = bufs[0].get_text(starts[0], ends[0], False)
                     text1 = text_type(text1, 'utf8')
                     textn = bufs[1].get_text(starts[1], ends[1], False)
                     textn = text_type(textn, 'utf8')
 
+                    bufs[0].delete_mark(start_marks[0])
+                    bufs[0].delete_mark(end_marks[0])
+                    bufs[1].delete_mark(start_marks[1])
+                    bufs[1].delete_mark(end_marks[1])
+
                     if texts != (text1, textn):
                         return
 
@@ -1611,6 +1648,8 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 bufdata.filename = bufdata.label = os.path.abspath(filename)
                 bufdata.savefile = None
                 self.fileentry[pane].set_filename(bufdata.filename)
+                self.filelabel_toolitem[pane].set_visible(False)
+                self.fileentry_toolitem[pane].set_visible(True)
             else:
                 return False
 
@@ -1671,6 +1710,8 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             self.emit("file-changed", save_to)
             self.undosequence.checkpoint(buf)
             bufdata.update_mtime()
+            if pane == 1 and self.num_panes == 3:
+                self.meta['middle_saved'] = True
             return True
         else:
             return False
