@@ -19,7 +19,6 @@ from __future__ import print_function
 import optparse
 import os
 import StringIO
-from gettext import gettext as _
 
 from gi.repository import Gio
 from gi.repository import GLib
@@ -30,6 +29,8 @@ import meld.conf
 import meld.preferences
 import meld.ui.util
 
+from meld.conf import _
+
 
 class MeldApp(Gtk.Application):
 
@@ -100,7 +101,11 @@ class MeldApp(Gtk.Application):
         meld.preferences.PreferencesDialog(self.get_active_window())
 
     def help_callback(self, action, parameter):
-        Gtk.show_uri(Gdk.Screen.get_default(), "help:meld",
+        if meld.conf.UNINSTALLED:
+            uri = "http://meldmerge.org/help/"
+        else:
+            uri = "help:meld"
+        Gtk.show_uri(Gdk.Screen.get_default(), uri,
                      Gtk.get_current_event_time())
 
     def about_callback(self, action, parameter):
@@ -125,10 +130,13 @@ class MeldApp(Gtk.Application):
         window.widget.meldwindow = window
         return window
 
+    def get_meld_window(self):
+        return self.get_active_window().meldwindow
+
     def open_paths(self, paths, **kwargs):
         new_tab = kwargs.pop('new_tab')
         if new_tab:
-            window = self.get_active_window().meldwindow
+            window = self.get_meld_window()
         else:
             window = self.new_window()
         return window.open_paths(paths, **kwargs)
@@ -251,7 +259,7 @@ class MeldApp(Gtk.Application):
             comparison_file_path = os.path.expanduser(path)
             gio_file = Gio.File.new_for_path(comparison_file_path)
             try:
-                tab = self.window.append_recent(gio_file.get_uri())
+                tab = self.get_meld_window().append_recent(gio_file.get_uri())
             except (IOError, ValueError):
                 parser.error(_("Error reading saved comparison file"))
             return tab
@@ -275,7 +283,7 @@ class MeldApp(Gtk.Application):
                 tab.set_merge_output_file(options.outfile)
 
         if error:
-            if not self.window.has_pages():
+            if not self.get_meld_window().has_pages():
                 parser.error(error)
             else:
                 print(error)
