@@ -15,7 +15,6 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import os
-from gettext import gettext as _
 
 from gi.repository import Gdk
 from gi.repository import Gio
@@ -34,8 +33,8 @@ from .ui import gnomeglade
 from .ui import notebooklabel
 
 from .util.compat import string_types
+from meld.conf import _
 from meld.recent import recent_comparisons
-
 from meld.settings import interface_settings, settings
 
 
@@ -134,13 +133,13 @@ class MeldWindow(gnomeglade.Component):
                 None, True),
         )
         ui_file = gnomeglade.ui_file("meldapp-ui.xml")
-        self.actiongroup = Gtk.ActionGroup('MainActions')
+        self.actiongroup = Gtk.ActionGroup(name='MainActions')
         self.actiongroup.set_translation_domain("meld")
         self.actiongroup.add_actions(actions)
         self.actiongroup.add_toggle_actions(toggleactions)
 
-        recent_action = Gtk.RecentAction("Recent",  _("Open Recent"),
-                                         _("Open recent files"), None)
+        recent_action = Gtk.RecentAction(name="Recent",  label=_("Open Recent"),
+                                         tooltip=_("Open recent files"), stock_id=None)
         recent_action.set_show_private(True)
         recent_action.set_filter(recent_comparisons.recent_filter)
         recent_action.set_sort_type(Gtk.RecentSortType.MRU)
@@ -359,6 +358,8 @@ class MeldWindow(gnomeglade.Component):
         # Only update the window title if the current page is active
         if self.notebook.get_current_page() == self.notebook.page_num(page):
             self.widget.set_title(text + " - Meld")
+        if isinstance(text, unicode):
+            text = text.encode('utf8')
         self.notebook.child_set_property(page, "menu-label", text)
 
         actiongroup = self.tab_switch_actiongroup
@@ -494,7 +495,7 @@ class MeldWindow(gnomeglade.Component):
             self.ui.remove_action_group(self.tab_switch_actiongroup)
 
         self.tab_switch_merge_id = self.ui.new_merge_id()
-        self.tab_switch_actiongroup = Gtk.ActionGroup("TabSwitchActions")
+        self.tab_switch_actiongroup = Gtk.ActionGroup(name="TabSwitchActions")
         self.ui.insert_action_group(self.tab_switch_actiongroup)
         group = None
         current_page = self.notebook.get_current_page()
@@ -503,7 +504,7 @@ class MeldWindow(gnomeglade.Component):
             label = self.notebook.get_menu_label_text(page) or ""
             name = "SwitchTab%d" % i
             tooltip = _("Switch to this tab")
-            action = Gtk.RadioAction(name, label, tooltip, None, i)
+            action = Gtk.RadioAction(name=name, label=label, tooltip=tooltip, stock_id=None, value=i)
             action.join_group(group)
             group = action
             action.set_active(current_page == i)
@@ -599,13 +600,15 @@ class MeldWindow(gnomeglade.Component):
             doc.on_button_diff_clicked(None)
         return doc
 
-    def append_filediff(self, files,  merge_output=None):
+    def append_filediff(self, files,  merge_output=None, meta=None):
         assert len(files) in (1, 2, 3)
         doc = filediff.FileDiff(len(files))
         self._append_page(doc, "text-x-generic")
         doc.set_files(files)
         if merge_output is not None:
             doc.set_merge_output_file(merge_output)
+        if meta is not None:
+            doc.set_meta(meta)
         return doc
 
     def append_filemerge(self, files, merge_output=None):
@@ -618,7 +621,7 @@ class MeldWindow(gnomeglade.Component):
         return doc
 
     def append_diff(self, paths, auto_compare=False, auto_merge=False,
-                    merge_output=None):
+                    merge_output=None, meta=None):
         dirslist = [p for p in paths if os.path.isdir(p)]
         fileslist = [p for p in paths if os.path.isfile(p)]
         if dirslist and fileslist:
@@ -629,7 +632,8 @@ class MeldWindow(gnomeglade.Component):
         elif auto_merge:
             return self.append_filemerge(paths, merge_output=merge_output)
         else:
-            return self.append_filediff(paths, merge_output=merge_output)
+            return self.append_filediff(
+                paths, merge_output=merge_output, meta=meta)
 
     def append_vcview(self, location, auto_compare=False):
         doc = vcview.VcView()
@@ -679,7 +683,8 @@ class MeldWindow(gnomeglade.Component):
                 tab = self.append_vcview(a, auto_compare)
 
         elif len(paths) in (2, 3):
-            tab = self.append_diff(paths, auto_compare, auto_merge)
+            tab = self.append_diff(
+                paths, auto_compare=auto_compare, auto_merge=auto_merge)
         if tab:
             recent_comparisons.add(tab)
             if focus:
