@@ -19,7 +19,6 @@
 """
 
 import os
-from gettext import gettext as _
 import errno
 import shutil
 import re
@@ -28,6 +27,8 @@ import subprocess
 from gi.repository import GObject
 from gi.repository import Gtk
 
+from meld.conf import _
+
 
 if os.name != "nt":
     from select import select
@@ -39,7 +40,8 @@ else:
         return rlist, wlist, xlist
 
 
-def error_dialog(primary, secondary, parent=None, messagetype=Gtk.MessageType.ERROR):
+def error_dialog(
+        primary, secondary, parent=None, messagetype=Gtk.MessageType.ERROR):
     """A common error dialog handler for Meld
 
     This should only ever be used as a last resort, and for errors that
@@ -63,27 +65,60 @@ def error_dialog(primary, secondary, parent=None, messagetype=Gtk.MessageType.ER
     dialog.destroy()
 
 
+def modal_dialog(
+        primary, secondary, buttons, parent=None,
+        messagetype=Gtk.MessageType.WARNING):
+    """A common message dialog handler for Meld
+
+    This should only ever be used for interactions that must be resolved
+    before the application flow can continue.
+
+    Primary must be plain text. Secondary must be valid markup.
+    """
+
+    if not parent:
+        from meld.meldapp import app
+        parent = app.window.widget
+    elif not isinstance(parent, Gtk.Window):
+        parent = parent.get_toplevel()
+
+    dialog = Gtk.MessageDialog(
+        parent=parent,
+        flags=Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
+        type=messagetype,
+        buttons=Gtk.ButtonsType.NONE,
+        message_format=primary)
+    dialog.format_secondary_markup(secondary)
+
+    for label, response_id in buttons:
+        dialog.add_button(label, response_id)
+
+    response = dialog.run()
+    dialog.destroy()
+    return response
+
+
 def run_dialog( text, parent=None, messagetype=Gtk.MessageType.WARNING, buttonstype=Gtk.ButtonsType.OK, extrabuttons=()):
     """Run a dialog with text 'text'.
        Extra buttons are passed as tuples of (button label, response id).
     """
     escaped = GObject.markup_escape_text(text)
-    d = Gtk.MessageDialog(None,
+    d = Gtk.MessageDialog(
+        None,
         Gtk.DialogFlags.DESTROY_WITH_PARENT,
         messagetype,
-        buttonstype,
-        '<span weight="bold" size="larger">%s</span>' % escaped)
+        buttonstype)
+    markup = '<span weight="bold" size="larger">%s</span>' % escaped
+    d.set_markup(markup)
     if parent and isinstance(parent, Gtk.Window):
         d.set_transient_for(parent.widget.get_toplevel())
-    for b,rid in extrabuttons:
+    for b, rid in extrabuttons:
         d.add_button(b, rid)
     d.vbox.set_spacing(12)
     hbox = d.vbox.get_children()[0]
     hbox.set_spacing(12)
-    d.image.set_alignment(0.5, 0)
-    d.image.set_padding(12, 12)
-    d.label.set_use_markup(1)
-    d.label.set_padding(12, 12)
+    d.props.image.set_alignment(0.5, 0)
+    d.props.image.set_padding(12, 12)
     ret = d.run()
     d.destroy()
     return ret
