@@ -14,14 +14,13 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from gettext import gettext as _
-
 from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Gtk
 from gi.repository import GtkSource
 
+from meld.conf import _
 from meld.filters import FilterEntry
 from meld.settings import settings
 from meld.ui.gnomeglade import Component
@@ -33,7 +32,7 @@ class FilterList(ListWidget):
     def __init__(self, key, filter_type):
         default_entry = [_("label"), False, _("pattern"), True]
         ListWidget.__init__(self, "EditableList.ui",
-                            "list_alignment", ["EditableListStore"],
+                            "list_vbox", ["EditableListStore"],
                             "EditableList", default_entry)
         self.key = key
         self.filter_type = filter_type
@@ -159,12 +158,22 @@ class GSettingsBoolComboBox(GSettingsComboBox):
     gsettings_value = GObject.property(type=bool, default=False)
 
 
+class GSettingsStringComboBox(GSettingsComboBox):
+
+    __gtype_name__ = "GSettingsStringComboBox"
+
+    gsettings_column = GObject.property(type=int, default=0)
+    gsettings_value = GObject.property(type=str, default="")
+
+
 class PreferencesDialog(Component):
 
     def __init__(self, parent):
         Component.__init__(self, "preferences.ui", "preferencesdialog",
                            ["adjustment1", "adjustment2", "fileorderstore",
-                            "sizegroup_editor", "timestampstore"])
+                            "sizegroup_editor", "timestampstore",
+                            "mergeorderstore", "sizegroup_file_order_labels",
+                            "sizegroup_file_order_combos"])
         self.widget.set_transient_for(parent)
 
         bindings = [
@@ -211,16 +220,17 @@ class PreferencesDialog(Component):
         self.checkbutton_wrap_word.set_active(wrap_mode == Gtk.WrapMode.WORD)
 
         filefilter = FilterList("filename-filters", FilterEntry.SHELL)
-        self.file_filters_tab.pack_start(filefilter.widget, True, True, 0)
+        self.file_filters_vbox.pack_start(filefilter.widget, True, True, 0)
 
         textfilter = FilterList("text-filters", FilterEntry.REGEX)
-        self.text_filters_tab.pack_start(textfilter.widget, True, True, 0)
+        self.text_filters_vbox.pack_start(textfilter.widget, True, True, 0)
 
         columnlist = ColumnList("folder-columns")
         self.column_list_vbox.pack_start(columnlist.widget, True, True, 0)
 
         self.combo_timestamp.bind_to('folder-time-resolution')
         self.combo_file_order.bind_to('vc-left-is-local')
+        self.combo_merge_order.bind_to('vc-merge-file-order')
 
         self.widget.show()
 
