@@ -1,3 +1,5 @@
+# coding=UTF-8
+
 # Copyright (C) 2002-2006 Stephen Kennedy <stevek@gnome.org>
 # Copyright (C) 2010-2013 Kai Willadsen <kai.willadsen@gmail.com>
 #
@@ -17,12 +19,12 @@
 from __future__ import print_function
 
 import atexit
+import logging
 import tempfile
 import shutil
 import os
 import stat
 import sys
-from gettext import gettext as _
 
 from gi.repository import Gdk
 from gi.repository import GLib
@@ -40,9 +42,12 @@ from .ui import emblemcellrenderer
 from .ui import gnomeglade
 from .ui import vcdialogs
 
-from meld.settings import settings
+from meld.conf import _
+from meld.settings import settings, bind_settings
 from meld.vc import _null
 
+log = logging.getLogger(__name__)
+
 
 def _commonprefix(files):
     if len(files) != 1:
@@ -134,12 +139,19 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
 
     __gtype_name__ = "VcView"
 
+    __gsettings_bindings__ = (
+        ('vc-status-filters', 'status-filters'),
+        ('vc-left-is-local', 'left-is-local'),
+        ('vc-merge-file-order', 'merge-file-order'),
+    )
+
     status_filters = GObject.property(
         type=GObject.TYPE_STRV,
         nick="File status filters",
         blurb="Files with these statuses will be shown by the comparison.",
     )
     left_is_local = GObject.property(type=bool, default=False)
+    merge_file_order = GObject.property(type=str, default="local-merge-remote")
 
     # Map action names to VC commands and required arguments list
     action_vc_cmds_map = {
@@ -152,6 +164,11 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         "VcRevert": ("revert_command", ()),
     }
 
+    # Map for inter-tab command() calls
+    command_map = {
+        'resolve': 'resolve',
+    }
+
     state_actions = {
         "flatten": ("VcFlatten", None),
         "modified": ("VcShowModified", entry_modified),
@@ -164,6 +181,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         melddoc.MeldDoc.__init__(self)
         gnomeglade.Component.__init__(self, "vcview.ui", "vcview",
                                       ["VcviewActions", 'liststore_vcs'])
+        bind_settings(self)
 
         self.ui_file = gnomeglade.ui_file("vcview-ui.xml")
         self.actiongroup = self.VcviewActions
@@ -229,10 +247,6 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         self.combobox_vcs.add_attribute(cell, 'sensitive', 2)
         self.combobox_vcs.lock = False
 
-        settings.bind('vc-status-filters', self, 'status-filters',
-                      Gio.SettingsBindFlags.DEFAULT)
-        settings.bind('vc-left-is-local', self, 'left-is-local',
-                      Gio.SettingsBindFlags.DEFAULT)
         settings.bind('vc-console-visible',
                       self.actiongroup.get_action('VcConsoleVisible'),
                       'active', Gio.SettingsBindFlags.DEFAULT)
@@ -248,10 +262,22 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                 self.state_filters.append(s)
                 self.actiongroup.get_action(action_name).set_active(True)
 
+    def _set_external_action_sensitivity(self, focused):
+        try:
+            self.main_actiongroup.get_action("OpenExternal").set_sensitive(
+                focused)
+        except AttributeError:
+            pass
+
     def on_container_switch_in_event(self, ui):
-        melddoc.MeldDoc.on_container_switch_in_event(self, ui)
+        super(VcView, self).on_container_switch_in_event(ui)
+        self._set_external_action_sensitivity(True)
         self.scheduler.add_task(self.on_treeview_cursor_changed)
 
+    def on_container_switch_out_event(self, ui):
+        self._set_external_action_sensitivity(False)
+        super(VcView, self).on_container_switch_out_event(ui)
+
     def update_visible_columns(self):
         for data_id in self.column_name_map:
             col = self.treeview.get_column(self.column_name_map[data_id])
@@ -411,7 +437,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             entries = self.vc.listdir(path)
             entries = [e for e in entries if any(f(e) for f in filters)]
             for e in entries:
-                if e.isdir:
+                if e.isdir and e.state != tree.STATE_REMOVED:
                     try:
                         st = os.lstat(e.path)
                     # Covers certain unreadable symlink cases; see bgo#585895
@@ -427,13 +453,19 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                         symlinks_followed.add(key)
 
                     if flattened:
-                        todo.append((Gtk.TreePath.new_first(), e.path))
+                        if e.state != tree.STATE_IGNORED:
+                            # If directory state is changed, render it in
+                            # in flattened mode.
+                            if e.state != tree.STATE_NORMAL:
+                                child = self.model.add_entries(it, [e.path])
+                                self._update_item_state(child, e, path[prefixlen:])
+                            todo.append((Gtk.TreePath.new_first(), e.path))
                         continue
 
                 child = self.model.add_entries(it, [e.path])
-                self._update_item_state(child, e, path[prefixlen:])
-                if e.isdir:
+                if e.isdir and e.state != tree.STATE_IGNORED:
                     todo.append((self.model.get_path(child), e.path))
+                self._update_item_state(child, e, path[prefixlen:])
 
             if flattened:
                 root = Gtk.TreePath.new_first()
@@ -464,7 +496,8 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                 self.treeview.expand_row(path, False)
         else:
             path = self.model.value_path(it, 0)
-            self.run_diff(path)
+            if not self.model.is_folder(it, 0, path):
+                self.run_diff(path)
 
     def run_diff(self, path):
         if os.path.isdir(path):
@@ -472,17 +505,31 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             return
 
         left_is_local = self.props.left_is_local
+        basename = os.path.basename(path)
+        meta = {
+            'parent': self,
+            'prompt_resolve': False,
+        }
 
-        if self.vc.get_entry(path).state == tree.STATE_CONFLICT and \
+        # May have removed directories in list.
+        vc_entry = self.vc.get_entry(path)
+        if vc_entry and vc_entry.state == tree.STATE_CONFLICT and \
                 hasattr(self.vc, 'get_path_for_conflict'):
+            local_label = _(u"%s — local") % basename
+            remote_label = _(u"%s — remote") % basename
+
             # We create new temp files for other, base and this, and
             # then set the output to the current file.
-            if left_is_local:
+            if self.props.merge_file_order == "local-merge-remote":
                 conflicts = (tree.CONFLICT_THIS, tree.CONFLICT_MERGED,
                              tree.CONFLICT_OTHER)
+                meta['labels'] = (local_label, None, remote_label)
+                meta['tablabel'] = _(u"%s (local, merge, remote)") % basename
             else:
                 conflicts = (tree.CONFLICT_OTHER, tree.CONFLICT_MERGED,
                              tree.CONFLICT_THIS)
+                meta['labels'] = (remote_label, None, local_label)
+                meta['tablabel'] = _(u"%s (remote, merge, local)") % basename
             diffs = [self.vc.get_path_for_conflict(path, conflict=c)
                      for c in conflicts]
             temps = [p for p, is_temp in diffs if is_temp]
@@ -491,11 +538,21 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                 'auto_merge': False,
                 'merge_output': path,
             }
+            meta['prompt_resolve'] = True
         else:
+            remote_label = _(u"%s — repository") % basename
             comp_path = self.vc.get_path_for_repo_file(path)
             temps = [comp_path]
-            diffs = [path, comp_path] if left_is_local else [comp_path, path]
+            if left_is_local:
+                diffs = [path, comp_path]
+                meta['labels'] = (None, remote_label)
+                meta['tablabel'] = _(u"%s (working, repository)") % basename
+            else:
+                diffs = [comp_path, path]
+                meta['labels'] = (remote_label, None)
+                meta['tablabel'] = _(u"%s (repository, working)") % basename
             kwargs = {}
+        kwargs['meta'] = meta
 
         for temp_file in temps:
             os.chmod(temp_file, 0o444)
@@ -627,6 +684,19 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             self.refresh_partial(workdir)
         yield workdir, r
 
+    def has_command(self, command):
+        return command in self.command_map
+
+    def command(self, command, files):
+        if not self.has_command(command):
+            log.error("Couldn't understand command %s", command)
+
+        if not isinstance(files, list):
+            log.error("Invalid files argument to '%s': %r", command, files)
+
+        command = getattr(self.vc, self.command_map[command])
+        command(self._command, files)
+
     def _command(self, command, files, refresh=1, working_dir=None):
         """Run 'command' on 'files'.
         """
